<?php namespace EscapeWork\Manager\Medias\Providers;

use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;
use League\Glide\ServerFactory;
use EscapeWork\Manager\Medias\Repositories\ConfigRepository;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        $root = __DIR__ . '/../..';

        $this->publishes([
            $root . '/config/manager-medias.php'    => config_path('manager-medias.php'),
            $root . '/resources/assets/javascript'  => public_path('packages/manager-medias/javascript'),
            $root . '/resources/assets/stylesheets' => public_path('packages/manager-medias/stylesheets'),
            $root . '/database/migrations'          => 'database/migrations',
        ]);

        # loading views and translations from resources
        $this->loadViewsFrom($root . '/resources/views', 'manager-medias');
        $this->loadTranslationsFrom($root . '/resources/translations', 'manager-medias');
        $this->loadValidators();

        $this->loadConfigs();
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $root = __DIR__ . '/../..';

        $this->registerProviders();
        $this->createAliases();

        $this->registerGlideServer();
        $this->registerMiddlewares();

        $this->mergeConfigFrom(
            $root . '/config/manager-medias.php', 'manager-medias'
        );
    }

    /**
     * Creating Glide server and registering it to the containter
     * @return void
     */
    protected function registerGlideServer()
    {
        $this->app->singleton('League\Glide\Server', function ($app) {
            $filesystem = $app->make('Illuminate\Contracts\Filesystem\Filesystem');

            return ServerFactory::create([
                'source'             => $filesystem->getDriver(),
                'cache'              => $filesystem->getDriver(),
                'source_path_prefix' => 'files',
                'cache_path_prefix'  => 'images/.cache',
                'base_url'           => 'media',
            ]);
        });
    }

    /**
     * Registering middlewares
     * @return void
     */
    protected function registerMiddlewares()
    {
        $kernel = new \EscapeWork\Manager\Medias\Http\Kernel;

        foreach ($kernel->getMiddlewares() as $key => $middleware) {
            $this->app->router->middleware($key, $middleware);
        }
    }

    /**
     * Loading config singleton class
     * @return void
     */
    protected function loadConfigs()
    {
        $this->app->singleton('EscapeWork\Manager\Medias\Repositories\ConfigRepository', function ($app) {
            return new ConfigRepository;
        });
    }

    /**
     * Resolving validators
     * @return void
     */
    protected function loadValidators()
    {
        $this->app->validator->extend(
            'image_array',
            'EscapeWork\Manager\Medias\Validators\ImageArrayValidator@validate'
        );
    }

    /**
     * Registering third-party providers
     * @return void
     */
    protected function registerProviders()
    {
        $this->app->register('EscapeWork\Manager\Medias\Providers\RouteServiceProvider');
        $this->app->register('EscapeWork\LaravelUploader\Providers\LaravelUploaderServiceProvider');
        $this->app->register('Illuminate\Html\HtmlServiceProvider');

        if ($this->app['request']->is(config('manager.url') . '/*')) {
            $this->app->register('EscapeWork\Manager\Medias\Providers\EventServiceProvider');
        }
    }

    /**
     * Creating aliases for third-party packages
     * @return void
     */
    protected function createAliases()
    {
        $loader = AliasLoader::getInstance();
        $loader->alias('Html', 'Illuminate\Html\HtmlFacade');
        $loader->alias('Form', 'Illuminate\Html\FormFacade');
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [
            'EscapeWork\Manager\Medias\Providers\ManagerServiceProvider',
            'Illuminate\Html\HtmlServiceProvider',
        ];
    }

}
